@@ -1,13 +1,12 @@
 import logging
 import datetime
-import subprocess
 
-import os
 import re
 import dateutil.tz
 
+from . import px_exec_util
+
 if False:
-    import logging  # NOQA
     from typing import Set, Optional  # NOQA
 
 LOG = logging.getLogger(__name__)
@@ -135,14 +134,7 @@ def call_last():
     """
     Call last and return the result as one big string
     """
-    env = os.environ.copy()
-    if "LANG" in env:
-        del env["LANG"]
-
-    last = subprocess.Popen("last",
-                            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                            env=env)
-    return last.communicate()[0].decode('utf-8')
+    return px_exec_util.run(["last"])
 
 
 def _to_timestamp(string, now):
