@@ -1,13 +1,17 @@
+import logging
 import datetime
 import operator
-import subprocess
 
 import os
 import re
 import pwd
 import six
+import errno
+import subprocess
 import dateutil.tz
+
 from . import px_commandline
+from . import px_exec_util
 
 
 import sys
@@ -18,9 +22,13 @@ if sys.version_info.major >= 3:
     from typing import Text        # NOQA
     from typing import Optional    # NOQA
     from typing import List        # NOQA
+    from typing import Iterable    # NOQA
     from six import text_type      # NOQA
 
 
+LOG = logging.getLogger(__name__)
+
+
 # Match + group: " 7708 1 Mon Mar  7 09:33:11 2016  netbios 0.1 0:00.08  0.0 /usr/sbin/netbiosd hj"
 PS_LINE = re.compile(
     " *([0-9]+) +([0-9]+) +([A-Za-z0-9: ]+) +([^ ]+) +([0-9.]+) +([-0-9.:]+) +([0-9.]+) +(.*)")
@@ -42,6 +50,40 @@ uid_to_username_cache = {}  # type: Dict[int, Text]
 get_command_cache = {}  # type: Dict[Text, Text]
 
 
+def _parse_time(time_s):
+    # type: (Text)->datetime.datetime
+    """
+    Parse a local date from ps into a datetime.datetime object.
+
+    Example inputs:
+      "Wed Dec 16 12:41:43 2020"
+      "Sat Jan  9 14:20:34 2021"
+    """
+
+    zero_based_month = [
+        u"Jan",
+        u"Feb",
+        u"Mar",
+        u"Apr",
+        u"May",
+        u"Jun",
+        u"Jul",
+        u"Aug",
+        u"Sep",
+        u"Oct",
+        u"Nov",
+        u"Dec"
+    ].index(time_s[4:7])
+
+    day_of_month = int(time_s[8:10])
+    hour = int(time_s[11:13])
+    minute = int(time_s[14:16])
+    second = int(time_s[17:19])
+    year = int(time_s[20:24])
+
+    return datetime.datetime(year, zero_based_month + 1, day_of_month, hour, minute, second, tzinfo=TIMEZONE)
+
+
 class PxProcess(object):
     def __init__(self,
                  cmdline,   # type: Text
@@ -62,9 +104,16 @@ class PxProcess(object):
         self.command = self._get_command()  # type: text_type
         self.lowercase_command = self.command.lower()  # type: text_type
 
-        time = datetime.datetime.strptime(start_time_string.strip(), "%c")
-        self.start_time = time.replace(tzinfo=TIMEZONE)  # type: datetime.datetime
+        self.start_time = _parse_time(start_time_string.strip())
         self.age_seconds = (now - self.start_time).total_seconds()  # type: float
+        if self.age_seconds < 0:
+            LOG.error("Process age < 0: age_seconds=%r now=%r start_time=%r start_time_string=%r timezone=%r",
+                self.age_seconds,
+                now,
+                self.start_time,
+                start_time_string.strip(),
+                datetime.datetime.now(TIMEZONE).tzname())
+            assert False
         assert self.age_seconds >= 0
         self.age_s = seconds_to_str(self.age_seconds)  # type: text_type
 
@@ -169,19 +218,29 @@ class PxProcess(object):
 
     def get_sudo_user(self):
         """Retrieves the $SUDO_USER value for this process, or None if not set"""
-        env = os.environ.copy()
-        if "LANG" in env:
-            del env["LANG"]
-        ps = subprocess.Popen(["ps", "e", str(self.pid)],
-                              stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                              env=env)
-        stdout = ps.communicate()[0].decode('utf-8')
+        stdout = px_exec_util.run(["ps", "e", str(self.pid)])
         match = re.match(".* SUDO_USER=([^ ]+)", stdout, re.DOTALL)
         if not match:
             return None
 
         return match.group(1)
 
+    def is_alive(self):
+        try:
+            # Signal 0 has no effect
+            os.kill(self.pid, 0)
+        except OSError as e:
+            if e.errno == errno.ESRCH:
+                # No such process
+                return False
+
+            # Process found but something else went wrong
+            return True
+
+        # No problem, process was there
+        return True
+
+
 
 class PxProcessBuilder(object):
     def __init__(self):
@@ -226,19 +285,6 @@ class PxProcessBuilder(object):
         )
 
 
-def call_ps():
-    """
-    Call ps and return the result in an iterable of one output line per process
-    """
-    env = os.environ.copy()
-    if "LANG" in env:
-        del env["LANG"]
-    ps = subprocess.Popen(["ps", "-ax", "-o", "pid,ppid,lstart,uid,pcpu,time,%mem,command"],
-                          stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                          env=env)
-    return ps.communicate()[0].decode('utf-8').splitlines()[1:]
-
-
 def parse_time(timestring):
     # type: (Text) -> float
     """Convert a CPU time string returned by ps to a number of seconds"""
@@ -301,8 +347,11 @@ def ps_line_to_process(ps_line, now):
 
 
 def create_kernel_process(now):
-    # Fake a process 0, this one isn't returned by ps. More info about PID 0:
-    # https://en.wikipedia.org/wiki/Process_identifier
+    # type: (datetime.datetime) -> PxProcess
+    """
+    Fake a process 0, this one isn't returned by ps. More info about PID 0:
+    https://en.wikipedia.org/wiki/Process_identifier
+    """
     process_builder = PxProcessBuilder()
     process_builder.pid = 0
     process_builder.ppid = None
@@ -341,13 +390,17 @@ def resolve_links(processes, now):
         elif process.ppid is None:
             process.parent = None
         else:
-            process.parent = processes[process.ppid]
+            process.parent = processes.get(process.ppid)
+
+        if process.parent is not None:
             process.parent.children.add(process)
 
 
 def remove_process_and_descendants(processes, pid):
+    # type: (Dict[int, PxProcess], int) -> None
     process = processes[pid]
-    process.parent.children.remove(process)
+    if process.parent is not None:
+        process.parent.children.remove(process)
     toexclude = [process]
     while toexclude:
         process = toexclude.pop()
@@ -359,11 +412,38 @@ def remove_process_and_descendants(processes, pid):
 def get_all():
     # type: () -> List[PxProcess]
     processes = {}
-    ps_lines = call_ps()
-    now = datetime.datetime.now().replace(tzinfo=TIMEZONE)
-    for ps_line in ps_lines:
-        process = ps_line_to_process(ps_line, now)
-        processes[process.pid] = process
+
+    # NOTE: Both the full path to ps and "close_fds = False" are important
+    # because they enable the use of _posix_spawn()...
+    #
+    # https://github.com/python/cpython/blob/998ae1fa3fb05a790071217cf8f6ae3a928da13f/Lib/subprocess.py#L1715
+    #
+    # ... and avoids prematurely waiting for ps to produce 50000 bytes:
+    #
+    # https://github.com/python/cpython/blob/998ae1fa3fb05a790071217cf8f6ae3a928da13f/Lib/subprocess.py#L1796
+    #
+    # If you want to change this, try benchmark_proc_get_all.py and make sure
+    # you don't regress.
+    close_fds = False
+    command = ["/bin/ps", "-ax", "-o", "pid=,ppid=,lstart=,uid=,pcpu=,time=,%mem=,command="]
+
+    with open(os.devnull, 'w') as DEVNULL:
+        ps = subprocess.Popen(command,
+            stdin=DEVNULL,
+            stdout=subprocess.PIPE,
+            stderr=DEVNULL,
+            close_fds=close_fds,
+            env=px_exec_util.ENV)
+
+        stdout = ps.stdout
+        assert stdout
+        now = datetime.datetime.now().replace(tzinfo=TIMEZONE)
+        for ps_line in stdout:
+            process = ps_line_to_process(ps_line.decode('utf-8'), now)
+            processes[process.pid] = process
+
+        if ps.wait() != 0:
+            raise IOError("Exit code {} from {}".format(ps.returncode, command))
 
     resolve_links(processes, now)
     remove_process_and_descendants(processes, os.getpid())
@@ -372,11 +452,13 @@ def get_all():
 
 
 def order_best_last(processes):
+    # type: (Iterable[PxProcess]) -> List[PxProcess]
     """Returns process list ordered with the most interesting one last"""
     return sorted(processes, key=operator.attrgetter('score', 'cmdline'))
 
 
 def order_best_first(processes):
+    # type: (Iterable[PxProcess]) -> List[PxProcess]
     """Returns process list ordered with the most interesting one first"""
     ordered = sorted(processes, key=operator.attrgetter('cmdline'))
     ordered = sorted(ordered, key=operator.attrgetter('score'), reverse=True)
