@@ -3,25 +3,20 @@
 import sys
 import copy
 import time
-import errno
-import signal
-import select
 import logging
 import unicodedata
 
 import os
 from . import px_load
-from . import px_pager
 from . import px_process
 from . import px_terminal
-from . import px_load_bar
-from . import px_cpuinfo
+from . import px_meminfo
 from . import px_processinfo
 from . import px_launchcounter
+from . import px_process_menu
 
 if False:
     # For mypy PEP-484 static typing validation
-    import logging               # NOQA
     from typing import List      # NOQA
     from typing import Dict      # NOQA
     from typing import Union     # NOQA
@@ -30,27 +25,12 @@ if False:
 
 LOG = logging.getLogger(__name__)
 
-# Used for informing our getch() function that a window resize has occured
-SIGWINCH_PIPE = os.pipe()
-
-# We'll report window resize as this key having been pressed.
-#
-# NOTE: This must be detected as non-printable by handle_search_keypress().
-SIGWINCH_KEY = u'\x00'
-
 CMD_WHATEVER = -1
 CMD_QUIT = 1
 CMD_RESIZE = 2
 CMD_HANDLED = 3
 
-KEY_ESC = "\x1b"
-KEY_BACKSPACE = "\x1b[3~"
-KEY_DELETE = "\x7f"
-KEY_UPARROW = "\x1b[A"
-KEY_DOWNARROW = "\x1b[B"
-KEY_ENTER = "\x0d"
-
-SEARCH_PROMPT_ACTIVE = px_terminal.bold("Search (ENTER when done): ")
+SEARCH_PROMPT_ACTIVE = px_terminal.inverse_video("Search (ENTER when done): ")
 SEARCH_PROMPT_INACTIVE = "Search ('/' to edit): "
 SEARCH_CURSOR = px_terminal.inverse_video(" ")
 
@@ -78,23 +58,6 @@ last_process_poll = 0.0
 sort_by_memory = False
 
 
-class ConsumableString(object):
-    def __init__(self, string):
-        # type: (text_type) -> None
-        self._string = string
-
-    def __len__(self):
-        return len(self._string)
-
-    def consume(self, to_consume):
-        # type: (text_type) -> bool
-        if not self._string.startswith(to_consume):
-            return False
-
-        self._string = self._string[len(to_consume):]
-        return True
-
-
 def adjust_cpu_times(baseline, current):
     # type: (List[px_process.PxProcess], List[px_process.PxProcess]) -> List[px_process.PxProcess]
     """
@@ -139,62 +102,6 @@ def adjust_cpu_times(baseline, current):
     return list(pid2proc.values())
 
 
-def read_select(fds, timeout_seconds):
-    """Select on any of the fds becoming ready for read, retry on EINTR"""
-
-    # NOTE: If you change this method, you must test running "px --top" and
-    # resize the window in both Python 2 and Python 3.
-    while True:
-        try:
-            return select.select(fds, [], [], timeout_seconds)[0]
-        except OSError as ose:
-            # Python 3
-            if ose.errno == errno.EINTR:
-                # EINTR happens when the terminal window is resized by the user,
-                # just try again.
-                continue
-
-            # Non-EINTR exceptions are unexpected, help!
-            raise
-        except select.error as se:
-            # Python 2
-            if se.args[0] == errno.EINTR:
-                # EINTR happens when the terminal window is resized by the user,
-                # just try again.
-                continue
-
-            # Non-EINTR exceptions are unexpected, help!
-            raise
-
-
-def getch(timeout_seconds=0, fd=None):
-    # type: (int, int) -> Optional[ConsumableString]
-    """
-    Wait at most timeout_seconds for a character to become available on stdin.
-
-    Returns the character, or None on timeout.
-    """
-    if fd is None:
-        fd = sys.stdin.fileno()
-
-    can_read_from = (
-        read_select([fd, SIGWINCH_PIPE[0]], timeout_seconds))
-
-    # Read all(ish) bytes from the first ready-for-read stream. If more than one
-    # stream is ready, we'll catch the second one on the next call to this
-    # function, so just doing the first is fine.
-    for stream in can_read_from:
-        string = os.read(stream, 1234).decode("UTF-8")
-        if len(string) > 0:
-            return ConsumableString(string)
-
-        # A zero length response means we got EOF from one of the streams. This
-        # happens (at least) during testing.
-        continue
-
-    return None
-
-
 def get_notnone_cpu_time_seconds(proc):
     # type: (px_process.PxProcess) -> float
     seconds = proc.cpu_time_seconds
@@ -211,6 +118,25 @@ def get_notnone_memory_percent(proc):
     return 0
 
 
+def sort_by_cpu_usage(toplist):
+    # type(List[px_process.PxProcess]) -> List[px_process.PxProcess]
+    can_sort_by_time = False
+    for process in toplist:
+        if process.cpu_time_seconds:
+            can_sort_by_time = True
+            break
+
+    if can_sort_by_time:
+        # There is at least one > 0 time in the process list, so sorting by time
+        # will be of some use
+        return sorted(toplist, key=get_notnone_cpu_time_seconds, reverse=True)
+
+    # No > 0 time in the process list, try CPU percentage as an approximation of
+    # that. This should happen on the first iteration when ptop has just been
+    # launched.
+    return sorted(toplist, key=lambda process: process.cpu_percent or 0, reverse=True)
+
+
 def get_toplist(baseline,  # type: List[px_process.PxProcess]
                 current,   # type: List[px_process.PxProcess]
                 by_memory=False  # type: bool
@@ -218,13 +144,13 @@ def get_toplist(baseline,  # type: List[px_process.PxProcess]
     # type(...) -> List[px_process.PxProcess]
     toplist = adjust_cpu_times(baseline, current)
 
-    # Sort by CPU time used, then most interesting first
+    # Sort by interestingness last
     toplist = px_process.order_best_first(toplist)
     if by_memory:
         toplist = sorted(toplist, key=get_notnone_memory_percent, reverse=True)
     else:
         # By CPU time, this is the default
-        toplist = sorted(toplist, key=get_notnone_cpu_time_seconds, reverse=True)
+        toplist = sort_by_cpu_usage(toplist)
 
     return toplist
 
@@ -278,7 +204,6 @@ def get_line_to_highlight(toplist, max_process_count):
 
 
 def get_screen_lines(
-    load_bar,  # type: px_load_bar.PxLoadBar
     toplist,   # type: List[px_process.PxProcess]
     launchcounter,  # type: px_launchcounter.Launchcounter
     rows,      # type: int
@@ -294,7 +219,7 @@ def get_screen_lines(
         toplist = list(filter(lambda p: p.match(search, require_exact_user=False), toplist))
 
     # Hand out different amount of lines to the different sections
-    header_height = 2
+    header_height = 3  # System load, RAM load, empty line
     footer_height = 0
     cputop_minheight = 10
     if include_footer:
@@ -303,9 +228,13 @@ def get_screen_lines(
     # Print header
     load = px_load.get_load_values()
     loadstring = px_load.get_load_string(load)
-    loadbar = load_bar.get_bar(load=load[0], columns=40, text=loadstring)
+
+    meminfo = px_meminfo.get_meminfo()
     lines = [
-        u"System load: " + loadbar,
+        px_terminal.crop_ansi_string_at_length(
+            px_terminal.bold(u"Sysload: ") + loadstring, columns),
+        px_terminal.crop_ansi_string_at_length(
+            px_terminal.bold(u"RAM Use: ") + meminfo, columns),
         u""]
 
     # Create a launchers section
@@ -317,8 +246,9 @@ def get_screen_lines(
             # Add a section header
             launchlines = [
                 '',
-                px_terminal.bold(
-                    "Launched binaries, launch counts in (parentheses)")
+                px_terminal.crop_ansi_string_at_length(
+                    px_terminal.bold(
+                        "Launched binaries, launch counts in (parentheses)"), columns)
             ] + launchlines
 
             # Cut if we got too many lines
@@ -334,6 +264,10 @@ def get_screen_lines(
     max_process_count -= 1
 
     highlight_row = get_line_to_highlight(toplist, max_process_count)
+    global top_mode
+    if top_mode == MODE_SEARCH:
+        highlight_row = None
+
     highlight_column = u"CPUTIME"
     if sort_by_memory:
         highlight_column = u"RAM"
@@ -344,8 +278,14 @@ def get_screen_lines(
     # number of processes
     toplist_table_lines += rows * ['']
 
-    lines += [px_terminal.bold("Top CPU using processes")]
-    global top_mode
+    top_what = "CPU"
+    if sort_by_memory:
+        top_what = "memory"
+    lines += [
+        px_terminal.crop_ansi_string_at_length(
+            px_terminal.bold("Top " + top_what + " using processes"), columns)
+    ]
+
     if top_mode == MODE_SEARCH:
         lines += [SEARCH_PROMPT_ACTIVE + px_terminal.bold(search or "") + SEARCH_CURSOR]
     else:
@@ -368,7 +308,6 @@ def get_screen_lines(
 
 
 def redraw(
-    load_bar,  # type: px_load_bar.PxLoadBar
     toplist,   # type: List[px_process.PxProcess]
     launchcounter,  # type: px_launchcounter.Launchcounter
     rows,      # type: int
@@ -384,36 +323,14 @@ def redraw(
     """
     global search_string
     lines = get_screen_lines(
-        load_bar, toplist, launchcounter, rows, columns, include_footer,
+        toplist, launchcounter, rows, columns, include_footer,
         search=search_string)
 
-    clear_sequence = u""
-    if clear:
-        clear_sequence = px_terminal.CLEAR_SCREEN
-
-    # We need both \r and \n when the TTY is in tty.setraw() mode
-    writebytes((clear_sequence + u"\r\n".join(lines)).encode('utf-8'))
-
-
-def page_process_info(pid):
-    # type: (Optional[int]) -> None
-    if pid is None:
-        # Nothing selected, never mind
-        return
-
-    # Is this PID available?
-    processes = px_process.get_all()
-    process = px_processinfo.find_process_by_pid(pid, processes)
-    if not process:
-        # Process not available, never mind
-        return
-
-    with px_terminal.normal_display():
-        px_pager.page_process_info(process, processes)
+    px_terminal.draw_screen_lines(lines, clear)
 
 
 def handle_search_keypresses(key_sequence):
-    # type: (ConsumableString) -> None
+    # type: (px_terminal.ConsumableString) -> None
     global search_string
     global last_highlighted_row
     global last_highlighted_pid
@@ -426,17 +343,17 @@ def handle_search_keypresses(key_sequence):
     # return
 
     while len(key_sequence) > 0:
-        if key_sequence.consume(KEY_BACKSPACE):
+        if key_sequence.consume(px_terminal.KEY_BACKSPACE):
             search_string = search_string[:-1]
-        elif key_sequence.consume(KEY_DELETE):
+        elif key_sequence.consume(px_terminal.KEY_DELETE):
             search_string = search_string[:-1]
-        elif key_sequence.consume(KEY_UPARROW):
+        elif key_sequence.consume(px_terminal.KEY_UPARROW):
             last_highlighted_row -= 1
             last_highlighted_pid = None
-        elif key_sequence.consume(KEY_DOWNARROW):
+        elif key_sequence.consume(px_terminal.KEY_DOWNARROW):
             last_highlighted_row += 1
             last_highlighted_pid = None
-        elif key_sequence.consume(KEY_ENTER) or key_sequence._string == KEY_ESC:
+        elif key_sequence.consume(px_terminal.KEY_ENTER) or key_sequence._string == px_terminal.KEY_ESC:
             # Exit search mode
             global top_mode
             top_mode = MODE_BASE
@@ -448,7 +365,7 @@ def handle_search_keypresses(key_sequence):
     if len(key_sequence) == 0:
         return
 
-    if KEY_ESC in key_sequence._string:
+    if px_terminal.KEY_ESC in key_sequence._string:
         # Some special key, unprintable, unhandled, never mind
         return
 
@@ -469,7 +386,7 @@ def get_command(**kwargs):
     """
     Call getch() and interpret the results.
     """
-    input = getch(**kwargs)
+    input = px_terminal.getch(**kwargs)
     if input is None:
         return None
     assert len(input) > 0
@@ -483,14 +400,20 @@ def get_command(**kwargs):
     global last_highlighted_pid
     global sort_by_memory
     while len(input) > 0:
-        if input.consume(KEY_UPARROW):
+        if input.consume(px_terminal.KEY_UPARROW):
             last_highlighted_row -= 1
             last_highlighted_pid = None
-        elif input.consume(KEY_DOWNARROW):
+        elif input.consume(px_terminal.KEY_DOWNARROW):
             last_highlighted_row += 1
             last_highlighted_pid = None
-        elif input.consume(KEY_ENTER):
-            page_process_info(last_highlighted_pid)
+        elif input.consume(px_terminal.KEY_ENTER):
+            if last_highlighted_pid is None:
+                continue
+            processes = px_process.get_all()
+            process = px_processinfo.find_process_by_pid(last_highlighted_pid, processes)
+            if not process:
+                continue
+            px_process_menu.PxProcessMenu(process).start()
         elif input.consume(u'/'):
             global search_string
             top_mode = MODE_SEARCH
@@ -499,7 +422,7 @@ def get_command(**kwargs):
             sort_by_memory = not sort_by_memory
         elif input.consume(u'q'):
             return CMD_QUIT
-        elif input.consume(SIGWINCH_KEY):
+        elif input.consume(px_terminal.SIGWINCH_KEY):
             return CMD_RESIZE
         else:
             # Unable to consume anything, give up
@@ -508,24 +431,21 @@ def get_command(**kwargs):
     return CMD_WHATEVER
 
 
-def _top():
-    # type: () -> None
+def _top(search=""):
+    # type: (str) -> None
+
+    global search_string
+    search_string = search
 
-    physical, logical = px_cpuinfo.get_core_count()
-    load_bar = px_load_bar.PxLoadBar(physical, logical)
     baseline = px_process.get_all()
     current = baseline
     launchcounter = px_launchcounter.Launchcounter()
     while True:
         launchcounter.update(current)
-        window_size = px_terminal.get_window_size()
-        if window_size is None:
-            sys.stderr.write("Cannot find terminal window size, are you on a terminal?\r\n")
-            exit(1)
-        rows, columns = window_size
+        rows, columns = px_terminal.get_window_size()
         global sort_by_memory
         toplist = get_toplist(baseline, current, sort_by_memory)
-        redraw(load_bar, toplist, launchcounter, rows, columns)
+        redraw(toplist, launchcounter, rows, columns)
 
         command = get_command(timeout_seconds=1)
 
@@ -535,7 +455,7 @@ def _top():
                 # The idea here is that if you terminate with "q" you still
                 # probably want the heading line on screen. So just do another
                 # update with somewhat fewer lines, and you'll get just that.
-                redraw(load_bar, toplist, launchcounter, rows - 4, columns, include_footer=False)
+                redraw(toplist, launchcounter, rows - 4, columns, include_footer=False)
                 return
 
             command = get_command(timeout_seconds=0)
@@ -549,25 +469,16 @@ def _top():
             last_process_poll = now
 
 
-def sigwinch_handler(signum, frame):
-    """Handle window resize signals by telling our getch() function to return"""
-    # "r" for "refresh" perhaps? The actual letter doesn't matter, as long as it
-    # doesn't collide with anything with some meaning other than "please
-    # redraw".
-    os.write(SIGWINCH_PIPE[1], SIGWINCH_KEY.encode("utf-8"))
-
-
-def top():
-    # type: () -> None
+def top(search=""):
+    # type: (str) -> None
 
     if not sys.stdout.isatty():
         sys.stderr.write('Top mode only works on TTYs, try running just "px" instead.\n')
         exit(1)
 
-    signal.signal(signal.SIGWINCH, sigwinch_handler)
     with px_terminal.fullscreen_display():
         try:
-            _top()
+            _top(search=search)
         except Exception:
             LOG.exception("Running ptop failed")
 
