@@ -14,20 +14,26 @@ def test_get_core_count():
 def test_get_core_count_from_proc_cpuinfo():
     my_dir = os.path.dirname(__file__)
 
-    physical, logical = px_cpuinfo.get_core_count_from_proc_cpuinfo(
-        os.path.join(my_dir, "proc-cpuinfo-2p4l"))
+    info = px_cpuinfo.get_core_count_from_proc_cpuinfo(
+        os.path.join(my_dir, "proc-cpuinfo-2p4l.txt"))
+    assert info
+    physical, logical = info
     assert physical == 2
     assert logical == 4
 
-    physical, logical = px_cpuinfo.get_core_count_from_proc_cpuinfo(
-        os.path.join(my_dir, "proc-cpuinfo-1p1l"))
+    info = px_cpuinfo.get_core_count_from_proc_cpuinfo(
+        os.path.join(my_dir, "proc-cpuinfo-1p1l.txt"))
+    assert info
+    physical, logical = info
     assert physical == 1
     assert logical == 1
 
     # This one is from my cell phone, just to provide a weird corner case example of things
     # we may have to handle.
-    physical, logical = px_cpuinfo.get_core_count_from_proc_cpuinfo(
-        os.path.join(my_dir, "proc-cpuinfo-8p8l"))
+    info = px_cpuinfo.get_core_count_from_proc_cpuinfo(
+        os.path.join(my_dir, "proc-cpuinfo-8p8l.txt"))
+    assert info
+    physical, logical = info
     assert physical == 8
     assert logical == 8
 
@@ -45,3 +51,23 @@ def test_get_core_count_from_sysctl():
     assert physical >= 1
     assert logical >= physical
     assert logical % physical == 0
+
+
+def test_parse_sysctl_output():
+    result = px_cpuinfo.parse_sysctl_output([])
+    assert result == (None, None)
+
+    result = px_cpuinfo.parse_sysctl_output(['gurka'])
+    assert result == (None, None)
+
+    result = px_cpuinfo.parse_sysctl_output([
+        'hw.physicalcpu: 1',
+        'hw.logicalcpu: 2'
+    ])
+    assert result == (1, 2)
+
+    result = px_cpuinfo.parse_sysctl_output([
+        'hw.physicalcpu: 12',
+        'hw.logicalcpu: 34'
+    ])
+    assert result == (12, 34)
