@@ -276,9 +276,9 @@ def test_ipc_pipe_osx():
     ipc_map = testutils.create_ipc_map(2222, files)
     assert 1001 in ipc_map._get_other_end_pids(f1)
     assert 2222 in ipc_map._get_other_end_pids(f2)
-    assert len(ipc_map.keys()) == 1
+    assert len(list(ipc_map.keys())) == 1
 
 
-def test_create_fake_process_str():
-    assert str(px_ipc_map.create_fake_process(pid=45)) == "PID 45"
-    assert str(px_ipc_map.create_fake_process(name="Johan")) == "Johan"
+def test_peer_process_str():
+    assert str(px_ipc_map.PeerProcess(pid=45)) == "PID 45"
+    assert str(px_ipc_map.PeerProcess(name="Johan")) == "Johan"
