@@ -35,27 +35,6 @@ def test_create_process():
     assert test_me.age_seconds > 0
 
 
-def test_call_ps():
-    lines = list(px_process.call_ps())
-
-    # There should be at least 20 processes running on any single system. If
-    # there's a counter example, we'll just have to lower the 20 and document
-    # the counter example.
-    assert len(lines) > 20
-
-    for line in lines:
-        # 20 is an arbitrary lowest limit, if there are cases where this test
-        # fails even though it shouldn't, just document the reason here and
-        # lower the limit
-        assert len(line) > 20
-
-        # We want these lines to be unicode, this is needed if we have unicode
-        # chars in command lines
-        assert isinstance(line, six.text_type)
-
-    assert "COMMAND" not in lines[0]
-
-
 def test_ps_line_to_process_unicode():
     process = testutils.create_process(cputime="2:14.15")
 
@@ -192,6 +171,9 @@ def test_get_all_swedish():
     won't mess up our parsing.
     """
     os.environ["LANG"] = "sv_SE.UTF-8"
+    os.environ["LC_TIME"] = "sv_SE.UTF-8"
+    os.environ["LC_NUMERIC"] = "sv_SE.UTF-8"
+
     _test_get_all()
 
 
@@ -345,3 +327,15 @@ def test_uid_to_username():
     username = px_process.uid_to_username(456789)
     assert username == "456789"
     assert isinstance(username, six.text_type)
+
+
+def test_resolve_links():
+    UNKNOWN_PID = 1323532
+    p1 = testutils.create_process(pid=1, ppid=UNKNOWN_PID)
+    p2 = testutils.create_process(pid=2, ppid=1)
+    processes = { p1.pid: p1, p2.pid: p2 }
+    px_process.resolve_links(processes, testutils.now())
+
+    assert p1.parent is None
+    assert p2.parent is p1
+    assert p1.children == { p2 }
